""" A simple command line interface wrapper around cookiecutter which uses the template defined in this project. """
import json
from pathlib import Path
import sys

from typer import Typer, Context, echo
from pyfiglet import Figlet

from cookiecutter.main import cookiecutter


TEMPLATES_DIRECTORY = Path(__file__).parent / "templates"


app = Typer()


@app.command(
        context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def generate(
    ctx: Context,
    template: str = "default",
    overwrite: bool = False,
    data: str = None,
    no_input: bool = False
):
    """ Generate a new project using given template """
    if data:
        kwargs = json.loads(Path(data).read_text())
    else:
        kwargs = {}
    is_key = True
    for extra_arg in ctx.args:
        if is_key:
            key = extra_arg
            if key.startswith("--"):
                key = key[2:]
            elif key.startswith("-"):
                key = key[1:]
            is_key = False
            continue
        kwargs[key] = extra_arg
        is_key = True
    template_path = TEMPLATES_DIRECTORY / template
    if not template_path.is_dir():
        echo(f"Template not found: {template}", file=sys.stderr)
        sys.exit(1)
    echo(f"Creating new project using fastrepo {template} template.\n")
    cookiecutter(
        str(template_path),
        overwrite_if_exists=overwrite,
        extra_context=kwargs,
        no_input=no_input
    )
    f = Figlet(font='slant')
    echo("\n")
    echo(f.renderText('Fastrepo'))
    echo("""
    Learn more by reading the documentation at: https://gu-charbon.gitbook.io/fastrepo/.

    Do not forget to enable your environment before starting to develop:

        $ poetry shell

    Once you activated your environment, you can use the following command to list all available tasks:

        $ inv --list
    Or
        $ poetry run inv --list

    If you did not activate your environment.

    Happy coding!
    """)



#!/usr/bin/env python
"""Hooks ran after project generation."""
from distutils.util import strtobool
from pathlib import Path
from shutil import rmtree
from subprocess import check_call
import platform
import sys

IS_WINDOWS = platform.system() == "Windows"


def panic(error: Exception, msg_fmt: str, *args, **kwargs):
    print(msg_fmt.format(*args, **kwargs), file=sys.stderr)
    print(f"Error: {error}.", file=sys.stderr)
    sys.exit(1)


def init_git() -> None:
    """Init a git repository."""
    try:
        check_call(["git", "init"])
        check_call(["git", "checkout", "-b", "next"])
    except Exception as error:
        panic(error, "Error encountered during 'git init' post install hook. Do you have git installed ?")


def install_precommit() -> None:
    """Install pre-commit hooks."""
    try:
        check_call(["poetry", "run", "pre-commit", "install"], shell=IS_WINDOWS)
        check_call(["poetry", "run", "pre-commit", "install", "--hook-type", "commit-msg"], shell=IS_WINDOWS)
    except Exception as error:
        panic(error, "Error encountered during 'pre-commit' post-install hook")


def first_commit() -> None:
    """Commit after repository has been generated successfully."""
    try:
        check_call(["poetry", "run", "git", "add", "."], shell=IS_WINDOWS)
        check_call(["poetry", "run", "git", "commit", "-m", "chore: first commit"], shell=IS_WINDOWS)
    except Exception as error:
        panic(error, "Error encountered during 'git commit' post-install hook.")


def install_package() -> None:
    """Install package using poetry."""
    try:
        check_call(["poetry", "install"], shell=IS_WINDOWS)
    except Exception as error:
        panic(error, "Error encountered during 'poetry install' post-install hook.")


if __name__ == "__main__":
    init_git()
    install_package()
    install_precommit()
    first_commit()



""" A test to check that version respects semver convention. It also illustrates fixture usage. """
import re


def test_semver_version(version: str):
    """ Receive current version from fixture and check that value is a valid semver version.

    In a real test you might want to do some operation on the fixture value
    before asserting a condition.
    """
    # Recommended regexp for semver string: https://semver.org/#is-there-a-suggested-regular-expression-regex-to-check-a-semver-string
    SEMVER_REGEXP = "^(?P<major>0|[1-9]\d*)\.(?P<minor>0|[1-9]\d*)\.(?P<patch>0|[1-9]\d*)(?:-(?P<prerelease>(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"  # noqa: W605
    matched = re.search(SEMVER_REGEXP, version)
    # Make sure there is a match
    assert matched is not None
    # This is mainly for mypy
    assert matched.lastindex
    # This one checks that major, minor and patch digits are specified
    assert matched.lastindex >= 3
